/**********************************************
* Colors
**********************************************/
/**********************************************
* Fonts (font-family)
**********************************************/
/**********************************************
* Shared
**********************************************/
/**********************************************
* http://meyerweb.com/eric/tools/css/reset/
*   v2.0 | 20110126
*   License: none (public domain)
**********************************************/
html,
body,
div,
span,
applet,
object,
iframe,
h1,
h2,
h3,
h4,
h5,
h6,
p,
blockquote,
pre,
a,
abbr,
acronym,
address,
big,
cite,
code,
del,
dfn,
em,
img,
ins,
kbd,
q,
s,
samp,
small,
strike,
strong,
sub,
sup,
tt,
var,
b,
u,
i,
center,
dl,
dt,
dd,
ol,
ul,
li,
fieldset,
form,
label,
legend,
table,
caption,
tbody,
tfoot,
thead,
tr,
th,
td,
article,
aside,
canvas,
details,
embed,
figure,
figcaption,
footer,
header,
hgroup,
menu,
nav,
output,
ruby,
section,
summary,
time,
mark,
audio,
video {
  margin: 0;
  padding: 0;
  border: 0;
  font-size: 100%;
  font: inherit;
  vertical-align: baseline;
}
/* HTML5 display-role reset for older browsers */
article,
aside,
details,
figcaption,
figure,
footer,
header,
hgroup,
menu,
nav,
section {
  display: block;
}
body {
  line-height: 1;
}
ol,
ul {
  list-style: none;
}
blockquote,
q {
  quotes: none;
}
blockquote:before,
blockquote:after,
q:before,
q:after {
  content: '';
  content: none;
}
table {
  border-collapse: collapse;
  border-spacing: 0;
}
/**********************************************
* Default
**********************************************/
@font-face {
  font-family: 'FontAwesome';
  src: url('font-awesome/font-awesome-webfont.eot');
  src: url('font-awesome/font-awesome-webfont.eot') format('embedded-opentype'), url('font-awesome/font-awesome-webfont.woff2') format('woff2'), url('font-awesome/font-awesome-webfont.woff') format('woff'), url('font-awesome/font-awesome-webfont.ttf') format('truetype'), url('font-awesome/font-awesome-webfont.svg') format('svg');
  font-weight: normal;
  font-style: normal;
}
html {
  height: 100%;
  width: 100%;
  display: block;
  background-color: #FFFFFF;
  box-sizing: border-box;
}
*,
*:before,
*:after {
  box-sizing: inherit;
}
body {
  height: 100%;
  width: 100%;
  display: block;
  color: #1F1F1F;
  word-wrap: break-word;
}
body,
button,
input,
textarea,
select {
  font-family: 'Open Sans', Helvetica, Arial, sans-serif;
  font-size: 12pt;
  padding: 0;
  margin: 0;
  font-weight: 400;
  line-height: 1.5;
}
@media only screen and (min-width: 45em) and (max-width: 85em) {
  body,
  button,
  input,
  textarea,
  select {
    font-size: 11pt;
  }
}
@media not screen and (max-width: 85em) {
  body,
  button,
  input,
  textarea,
  select {
    font-size: 10pt;
  }
}
/**********************************************
* Inline elements
**********************************************/
strong {
  font-weight: bold;
}
i,
cite,
em,
var,
address,
dfn {
  font-style: italic;
}
strong,
b {
  font-weight: bold;
}
u,
ins {
  text-decoration: underline;
}
s,
strike,
del {
  text-decoration: line-through;
}
sup {
  vertical-align: super;
  font-size: smaller;
}
sub {
  vertical-align: sub;
  font-size: smaller;
}
small {
  font-size: smaller;
}
tt,
code,
kbd,
samp {
  font-family: "Courier New", Courier, monospace;
}
q {
  display: inline;
  quotes: initial;
}
q:before {
  content: open-quote;
}
q:after {
  content: close-quote;
}
mark {
  color: #22CC22;
}
/**********************************************
* Link
**********************************************/
a {
  color: #2FA2E4;
  text-decoration: none;
}
a[href=""] {
  color: #1F1F1F;
}
a:hover {
  text-decoration: underline;
}
a:focus {
  outline-offset: -2px;
  outline: 2px solid #3388cc;
}
@media not screen and (max-width: 85em) {
  a[accesskey]:after {
    content: " (" attr(accesskey) ") ";
  }
}
/**********************************************
* Button
**********************************************/
/**********************************************
* Preformatted
**********************************************/
pre,
xmp,
plaintext,
listing {
  display: block;
  white-space: pre-wrap;
}
/**********************************************
* hr
**********************************************/
hr {
  display: block;
  border-style: inset;
  border-width: 1px;
  border-color: #FF6600;
}
/**********************************************
* Text fields
**********************************************/
label {
  display: block;
}
input:not([type=submit]):not([type=file]):not([type=checkbox]):not([type=radio]):not([type=color]),
textarea,
select {
  width: 100%;
  padding: 3pt;
  background-color: #FFFFFF;
  color: #1F1F1F;
  border: 1px solid rgba(0, 0, 0, 0.3);
  border-radius: 0.325em;
  transition: border 0.2s ease-out, box-shadow 0.2s ease-out;
}
input:not([type=submit]):not([type=file]):not([type=checkbox]):not([type=radio]):not([type=color]):active,
textarea:active,
select:active,
input:not([type=submit]):not([type=file]):not([type=checkbox]):not([type=radio]):not([type=color]):focus,
textarea:focus,
select:focus {
  outline: none;
}
input:not([type=submit]):not([type=file]):not([type=checkbox]):not([type=radio]):not([type=color]):focus,
textarea:focus,
select:focus {
  border: 1px solid #3388cc;
  box-shadow: 0 0 12pt #3388cc;
}
input:not([type=submit]):not([type=file]):not([type=checkbox]):not([type=radio]):not([type=color]):invalid,
textarea:invalid,
select:invalid,
input:not([type=submit]):not([type=file]):not([type=checkbox]):not([type=radio]):not([type=color]).is-invalid,
textarea.is-invalid,
select.is-invalid {
  border: 1px solid #FF6600;
}
input:not([type=submit]):not([type=file]):not([type=checkbox]):not([type=radio]):not([type=color]):invalid:focus,
textarea:invalid:focus,
select:invalid:focus,
input:not([type=submit]):not([type=file]):not([type=checkbox]):not([type=radio]):not([type=color]).is-invalid:focus,
textarea.is-invalid:focus,
select.is-invalid:focus {
  box-shadow: 0 0 12pt #FF6600;
}
label.is-invalid {
  color: #FF6600;
}
input[type="search"] {
  -webkit-appearance: textfield;
}
input[type="search"]::-webkit-search-decoration {
  -webkit-appearance: none;
}
select {
  cursor: pointer;
  -webkit-appearance: none;
  -moz-appearance: none;
  appearance: none;
  background-color: #FFFFFF;
  padding-right: 24pt;
  background-image: url("data:image/svg+xml;utf8,<svg xmlns='http://www.w3.org/2000/svg' width='50px' height='50px'><polyline fill-opacity='0.5' points='46.139,15.518 25.166,36.49 4.193,15.519'/></svg>");
  background-position: right 6pt top 50%;
  background-repeat: no-repeat;
  background-size: 12pt 12pt;
}
select:-moz-focusring {
  color: transparent;
  text-shadow: 0 0 0 #000;
}
textarea {
  word-wrap: break-word;
  white-space: pre-wrap;
  vertical-align: top;
  transition: height 0.2s ease-out;
}
@media not screen and (min-width: 45em) {
  textarea {
    height: 4em;
  }
  textarea:focus {
    height: 20em;
  }
}
@media only screen and (min-width: 45em) and (max-width: 85em), not screen and (max-width: 85em) {
  textarea {
    min-height: 10em;
  }
}
::-webkit-input-placeholder {
  color: #575757;
}
:-moz-placeholder {
  /* Firefox 18- */
  color: #575757;
}
::-moz-placeholder {
  /* Firefox 19+ */
  color: #575757;
}
:-ms-input-placeholder {
  color: #575757;
}
input[type=submit],
button {
  margin: 0;
  padding: 0;
  border: none;
  background: transparent;
  color: #1F1F1F;
  /*
  // XXX TODO
  // transition: background-color 0.2s ease-out;
  &:hover, &:focus {
    outline: none;
  }
  &:active {
    // box-shadow: inset 0 0 0 5px darken(@colorblocklinkbackground, 10%);
    background-color: darken(@colorblocklinkbackground, 20%);
  }
*/
  cursor: pointer;
}
input[type=submit]::-moz-focus-inner,
button::-moz-focus-inner {
  border: 0px;
  padding: 0px;
}
input[type=submit]:hover,
button:hover,
input[type=submit]:focus,
button:focus {
  outline: none;
}
/**********************************************
* Media
**********************************************/
img,
iframe,
video,
svg {
  max-width: 100%;
}
svg polyline {
  stroke: #1F1F1F;
  fill: none;
}
svg text {
  stroke: #1F1F1F;
  fill: #1F1F1F;
}
iframe {
  width: 100%;
  height: 30em;
  background-color: #FFFFFF;
}
img {
  image-orientation: from-image;
}
.ui-info-error {
  color: red;
}
/**********************************************
* Gadget: panel
**********************************************/
div[data-gadget-scope='panel'] {
  background-color: #393939;
  color: #FFFFFF;
  width: 180pt;
  min-height: 100%;
  max-height: 100%;
  overflow-y: auto;
  position: fixed;
  top: 0;
  left: 0;
  display: block;
  z-index: 2000;
  transition: transform 200ms ease-out;
  transform: translate3d(0, 0, 0);
}
@media not screen and (min-width: 45em), only screen and (min-width: 45em) and (max-width: 85em) {
  div[data-gadget-scope='panel'] {
    box-shadow: 5px 0 5px rgba(0, 0, 0, 0.15);
  }
}
@media not screen and (min-width: 45em), only screen and (min-width: 45em) and (max-width: 85em) {
  div[data-gadget-scope='panel'] {
    left: -186pt;
  }
  div[data-gadget-scope='panel'].visible {
    transform: translate3d(186pt, 0, 0);
  }
}
div[data-gadget-scope='panel'] div[data-role="header"] {
  display: flex;
  justify-content: flex-start;
}
div[data-gadget-scope='panel'] div[data-role="header"] .panel_img {
  text-align: center;
  flex: 1;
  height: 30pt;
}
div[data-gadget-scope='panel'] div[data-role="header"] button,
div[data-gadget-scope='panel'] div[data-role="header"] a {
  width: 3em;
  overflow: hidden;
  text-indent: -9999px;
  white-space: nowrap;
  background-color: #393939;
  display: block;
  line-height: 30pt;
  color: #FFFFFF;
}
div[data-gadget-scope='panel'] div[data-role="header"] button::before,
div[data-gadget-scope='panel'] div[data-role="header"] a::before {
  float: left;
  text-indent: 0;
  margin-left: 12pt;
}
@media not screen and (max-width: 85em) {
  div[data-role='page']:not(.desktop-panel-hidden) div[data-gadget-scope='panel'] div[data-role="header"] button[data-i18n="Close"],
  div[data-role='page']:not(.desktop-panel-hidden) div[data-gadget-scope='panel'] div[data-role="header"] a[data-i18n="Close"] {
    display: none;
  }
}
div[data-gadget-scope='panel'] div[data-gadget-scope='erp5_searchfield'] {
  padding: 3pt 12pt;
}
div[data-gadget-scope='panel'] div[data-gadget-scope='erp5_searchfield'] button {
  color: #FFFFFF;
}
div[data-gadget-scope='panel'] div[data-gadget-scope='erp5_searchfield'] input[type="search"] {
  color: #FFFFFF !important;
  background-color: #767676 !important;
}
div[data-gadget-scope='panel'] img {
  text-align: left;
  height: 100%;
}
div[data-gadget-scope='panel'] div[data-gadget-scope="erp5_checkbox"] {
  padding: 3pt;
  padding-left: 12pt;
}
div[data-gadget-scope='panel'] div[data-gadget-scope="erp5_checkbox"] label,
div[data-gadget-scope='panel'] div[data-gadget-scope="erp5_checkbox"] span,
div[data-gadget-scope='panel'] div[data-gadget-scope="erp5_checkbox"] input {
  cursor: pointer;
}
div[data-gadget-scope='panel'] div[data-gadget-scope="erp5_checkbox"] span {
  width: 24pt;
  display: inline-block;
}
div[data-gadget-scope='panel'] ul:first-child {
  margin-top: 30pt;
}
div[data-gadget-scope='panel'] ul li a {
  color: #FFFFFF;
  display: block;
  padding: 1.5pt;
  padding-left: 12pt;
  text-overflow: ellipsis;
  overflow: hidden;
  white-space: nowrap;
}
div[data-gadget-scope='panel'] ul li a.active {
  color: #1F1F1F;
  background-color: #FFFFFF;
}
div[data-gadget-scope='panel'] ul li a:hover,
div[data-gadget-scope='panel'] ul li a:active {
  color: #FFFFFF;
  background-color: #1F1F1F;
}
div[data-gadget-scope='panel'] ul li a::before {
  width: 24pt;
}
div[data-gadget-scope='panel'] dl {
  background-color: #767676;
  color: #1F1F1F;
  transform-origin: 50% 0;
  transform: scaleY(0);
  transition: transform 0.2s ease-out;
}
div[data-gadget-scope='panel'] dl:not(:empty) {
  transform: scaleY(1);
}
div[data-gadget-scope='panel'] dl dt {
  padding: 1.5pt;
  padding-left: 12pt;
  text-overflow: ellipsis;
  overflow: hidden;
  white-space: nowrap;
  text-transform: uppercase;
}
div[data-gadget-scope='panel'] dl dt::before {
  width: 24pt;
}
div[data-gadget-scope='panel'] dl dd a {
  color: #FFFFFF;
  display: block;
  padding: 1.5pt;
  padding-left: 12pt;
  text-overflow: ellipsis;
  overflow: hidden;
  white-space: nowrap;
  padding-left: 36pt;
}
div[data-gadget-scope='panel'] dl dd a.active {
  color: #1F1F1F;
  background-color: #FFFFFF;
}
div[data-gadget-scope='panel'] dl dd a:hover,
div[data-gadget-scope='panel'] dl dd a:active {
  color: #1F1F1F;
  background-color: #FFFFFF;
}
div[data-gadget-scope='panel'] dl dd a::before {
  width: 24pt;
}
/**********************************************
* Gadget: editor panel
**********************************************/
div[data-gadget-scope='editor_panel'] {
  background-color: #FFFFFF;
  width: 180pt;
  min-height: 100%;
  max-height: 100%;
  overflow-y: auto;
  position: fixed;
  top: 0;
  display: block;
  z-index: 3000;
}
@media not screen and (max-width: 85em) {
  div[data-gadget-scope='editor_panel'] {
    left: -186pt;
    transition: transform 200ms ease-out;
    transform: translate3d(0, 0, 0);
    box-shadow: 5px 0 5px rgba(0, 0, 0, 0.15);
  }
  div[data-gadget-scope='editor_panel'].visible {
    transform: translate3d(186pt, 0, 0);
  }
}
@media not screen and (min-width: 45em), only screen and (min-width: 45em) and (max-width: 85em) {
  div[data-gadget-scope='editor_panel'] {
    right: -186pt;
    transition: transform 200ms ease-out;
    transform: translate3d(0, 0, 0);
    box-shadow: -5px 0 5px rgba(0, 0, 0, 0.15);
  }
  div[data-gadget-scope='editor_panel'].visible {
    transform: translate3d(-186pt, 0, 0);
  }
}
div[data-gadget-scope='editor_panel'] div[data-role="header"] {
  display: flex;
  justify-content: space-between;
  flex-direction: row-reverse;
}
div[data-gadget-scope='editor_panel'] div[data-role="header"] h1 {
  text-align: left;
  line-height: 30pt;
  max-height: 30pt;
}
div[data-gadget-scope='editor_panel'] div[data-role="header"] button,
div[data-gadget-scope='editor_panel'] div[data-role="header"] a {
  width: 3em;
  overflow: hidden;
  text-indent: -9999px;
  white-space: nowrap;
  display: block;
  line-height: 30pt;
}
div[data-gadget-scope='editor_panel'] div[data-role="header"] button::before,
div[data-gadget-scope='editor_panel'] div[data-role="header"] a::before {
  float: left;
  text-indent: 0;
  margin-left: 12pt;
}
div[data-gadget-scope='editor_panel'] section {
  padding: 12pt;
}
div[data-gadget-scope='editor_panel'] section fieldset > div {
  display: inline-block;
}
div[data-gadget-scope='editor_panel'] section fieldset label {
  display: inline-block;
  text-align: center;
}
div[data-gadget-scope='editor_panel'] section fieldset input[type="radio"] {
  display: inline-block;
}
div[data-gadget-scope='editor_panel'] section .filter_item_container > div,
div[data-gadget-scope='editor_panel'] section .sort_item_container > div,
div[data-gadget-scope='editor_panel'] section .column_item_container > div {
  display: flex;
  align-items: flex-start;
  padding: 6pt 0;
}
div[data-gadget-scope='editor_panel'] section .filter_item_container > div .filter_item,
div[data-gadget-scope='editor_panel'] section .sort_item_container > div .filter_item,
div[data-gadget-scope='editor_panel'] section .column_item_container > div .filter_item,
div[data-gadget-scope='editor_panel'] section .filter_item_container > div .sort_item,
div[data-gadget-scope='editor_panel'] section .sort_item_container > div .sort_item,
div[data-gadget-scope='editor_panel'] section .column_item_container > div .sort_item,
div[data-gadget-scope='editor_panel'] section .filter_item_container > div .column_item,
div[data-gadget-scope='editor_panel'] section .sort_item_container > div .column_item,
div[data-gadget-scope='editor_panel'] section .column_item_container > div .column_item {
  flex: 1;
}
div[data-gadget-scope='editor_panel'] section button {
  padding: 3pt 6pt;
  border: 1px solid rgba(0, 0, 0, 0.14);
  border-radius: 0.325em;
  margin-right: 6pt;
  width: 2em;
  overflow: hidden;
  text-indent: -9999px;
  white-space: nowrap;
}
div[data-gadget-scope='editor_panel'] section button:last-of-type {
  margin-right: 0;
}
div[data-gadget-scope='editor_panel'] section button::before {
  margin-right: 6pt;
  float: left;
  text-indent: 0;
}
div[data-gadget-scope='editor_panel'] section button.trash {
  float: right;
}
/**********************************************
* Gadget: header
**********************************************/
div[data-gadget-scope='header'] .ui-header {
  position: fixed;
  z-index: 1000;
  text-align: center;
  display: flex;
  flex-flow: row wrap;
  width: 100%;
  color: #FFFFFF;
  text-shadow: 0 1px 2px rgba(0, 0, 0, 0.2);
  background-color: #0E81C2;
}
@media not screen and (max-width: 85em) {
  div[data-gadget-scope='header'] .ui-header {
    transition: transform 200ms ease-out;
    transform: translate3d(180pt, 0, 0);
  }
}
div[data-gadget-scope='header'] .ui-header button,
div[data-gadget-scope='header'] .ui-header a {
  color: #FFFFFF;
  text-shadow: 0 1px 2px rgba(0, 0, 0, 0.2);
  transition: background-color 0.2s ease-out;
}
div[data-gadget-scope='header'] .ui-header button:hover,
div[data-gadget-scope='header'] .ui-header a:hover,
div[data-gadget-scope='header'] .ui-header button:active,
div[data-gadget-scope='header'] .ui-header a:active {
  background-color: #0e90d8;
}
div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls button,
div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls a {
  display: block;
  transition: background-color 0.2s ease-out;
  line-height: 30pt;
}
@media not screen and (max-width: 85em) {
  div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls button,
  div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls a {
    background-color: #0E81C2;
  }
}
@media not screen and (min-width: 45em) {
  div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls button,
  div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls a {
    background-color: #085078;
    width: 3em;
    overflow: hidden;
    text-indent: -9999px;
    white-space: nowrap;
  }
}
div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls button:hover,
div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls a:hover,
div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls button:active,
div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls a:active {
  background-color: #0e90d8;
}
@media only screen and (min-width: 45em) and (max-width: 85em) {
  div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls button,
  div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls a {
    background-color: #085078;
    width: 8em;
  }
}
@media only screen and (min-width: 45em) and (max-width: 85em), not screen and (max-width: 85em) {
  div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls button::before,
  div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls a::before {
    margin-right: 6pt;
    text-align: center;
    width: 1em;
  }
}
@media not screen and (min-width: 45em) {
  div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls button::before,
  div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls a::before {
    float: left;
    text-indent: 0;
    margin-left: 12pt;
  }
}
@media not screen and (max-width: 85em) {
  div[data-role='page']:not(.desktop-panel-hidden) div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls button[name="panel"],
  div[data-role='page']:not(.desktop-panel-hidden) div[data-gadget-scope='header'] .ui-header .ui-controlgroup-controls a[name="panel"] {
    display: none;
  }
}
div[data-gadget-scope='header'] .ui-header > .ui-btn-left button,
div[data-gadget-scope='header'] .ui-header > .ui-btn-left a {
  border-right: 1px solid rgba(255, 255, 255, 0.55);
}
@media not screen and (min-width: 45em), only screen and (min-width: 45em) and (max-width: 85em) {
  div[data-gadget-scope='header'] .ui-header > .ui-btn-right button,
  div[data-gadget-scope='header'] .ui-header > .ui-btn-right a {
    border-left: 1px solid rgba(255, 255, 255, 0.55);
  }
}
@media not screen and (max-width: 85em) {
  div[data-role='page']:not(.desktop-panel-hidden) div[data-gadget-scope='header'] .ui-header > .ui-btn-right button,
  div[data-role='page']:not(.desktop-panel-hidden) div[data-gadget-scope='header'] .ui-header > .ui-btn-right a {
    padding-left: 24pt;
    padding-right: 24pt;
    min-width: 5em;
  }
}
div[data-gadget-scope='header'] .ui-header > .ui-btn-right button.ui-icon-warning,
div[data-gadget-scope='header'] .ui-header > .ui-btn-right a.ui-icon-warning {
  background-color: #FF6600;
}
div[data-gadget-scope='header'] .ui-header h1 {
  text-align: left;
  line-height: 30pt;
  flex: 1;
  background-color: #085078;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
}
@media not screen and (max-width: 85em) {
  div[data-role='page']:not(.desktop-panel-hidden) div[data-gadget-scope='header'] .ui-header h1 {
    flex: 1 100%;
  }
}
div[data-gadget-scope='header'] .ui-header h1 > span {
  padding-left: 24pt;
}
div[data-gadget-scope='header'] .ui-header h1 > span::before {
  width: 1em;
  margin-right: 6pt;
}
@media only screen and (min-width: 45em) and (max-width: 85em) {
  div[data-gadget-scope='header'] .ui-header h1 > span {
    padding-left: 16pt;
  }
  div[data-gadget-scope='header'] .ui-header h1 > span::before {
    margin-right: 14pt;
  }
}
@media not screen and (min-width: 45em) {
  div[data-gadget-scope='header'] .ui-header h1 > span {
    padding-left: 12pt;
  }
  div[data-gadget-scope='header'] .ui-header h1 > span::before {
    margin-right: 6pt;
  }
}
div[data-gadget-scope='header'] .ui-header h1 form {
  height: 100%;
}
div[data-gadget-scope='header'] .ui-header h1 form button {
  text-align: left;
  height: 100%;
  width: 100%;
}
div[data-gadget-scope='header'] .ui-header h1 a,
div[data-gadget-scope='header'] .ui-header h1 button {
  display: block;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
}
div[data-gadget-scope='header'] .ui-header h1 a::before,
div[data-gadget-scope='header'] .ui-header h1 button::before {
  display: inline-block;
  width: 42pt;
}
@media only screen and (min-width: 45em) and (max-width: 85em), not screen and (min-width: 45em) {
  div[data-gadget-scope='header'] .ui-header h1 a::before,
  div[data-gadget-scope='header'] .ui-header h1 button::before {
    text-align: center;
  }
}
@media not screen and (max-width: 85em) {
  div[data-role='page']:not(.desktop-panel-hidden) div[data-gadget-scope='header'] .ui-header h1 a::before,
  div[data-role='page']:not(.desktop-panel-hidden) div[data-gadget-scope='header'] .ui-header h1 button::before {
    width: 12pt;
    margin-left: 24pt;
    margin-right: 6pt;
  }
}
@media not screen and (min-width: 45em) {
  div[data-gadget-scope='header'] .ui-header h1 a::before,
  div[data-gadget-scope='header'] .ui-header h1 button::before {
    width: 30pt;
  }
}
@media not screen and (max-width: 85em) {
  div[data-gadget-scope='header'] .ui-header .ui-subheader {
    text-align: left;
    flex: 1;
  }
}
@media not screen and (min-width: 45em), only screen and (min-width: 45em) and (max-width: 85em) {
  div[data-gadget-scope='header'] .ui-header .ui-subheader {
    flex: 1 100%;
  }
}
div[data-gadget-scope='header'] .ui-header ul {
  display: flex;
}
@media only screen and (min-width: 45em) and (max-width: 85em), not screen and (min-width: 45em) {
  div[data-gadget-scope='header'] .ui-header ul li {
    flex: 1;
    border-left: 1px solid rgba(0, 0, 0, 0.55);
  }
  div[data-gadget-scope='header'] .ui-header ul li:first-child {
    border-left: none;
  }
  div[data-gadget-scope='header'] .ui-header ul li a {
    display: block;
  }
}
@media only screen and (min-width: 45em) and (max-width: 85em) {
  div[data-gadget-scope='header'] .ui-header ul li a {
    padding-top: 3pt;
    padding-bottom: 3pt;
    line-height: inherit;
    white-space: nowrap;
    overflow: hidden;
  }
  div[data-gadget-scope='header'] .ui-header ul li a::before {
    font-size: 1.2em;
    display: block;
  }
}
@media not screen and (min-width: 45em) {
  div[data-gadget-scope='header'] .ui-header ul li a {
    text-align: center;
    vertical-align: middle;
    font-size: 1.5em;
    padding-top: 6pt;
    padding-bottom: 6pt;
    overflow: hidden;
    text-indent: -9999px;
    white-space: nowrap;
  }
  div[data-gadget-scope='header'] .ui-header ul li a::before {
    float: left;
    text-indent: 0;
    width: 100%;
  }
}
@media not screen and (max-width: 85em) {
  div[data-gadget-scope='header'] .ui-header ul li a {
    display: block;
    line-height: 30pt;
    white-space: nowrap;
    overflow: hidden;
  }
  div[data-role='page']:not(.desktop-panel-hidden) div[data-gadget-scope='header'] .ui-header ul li a {
    padding-left: 24pt;
    padding-right: 24pt;
    min-width: 5em;
  }
  div[data-gadget-scope='header'] .ui-header ul li a::before {
    margin-right: 6pt;
    text-align: center;
    width: 1em;
  }
}
/**********************************************
* Gadget: main
**********************************************/
.gadget-content {
  padding: 24pt;
  padding-top: 66pt;
  /*
  @media @smartphone {
    .ui-field-contain {
      // padding: 0.8em 0;
      // make sure there is a bottom border
      // XXX TODO: border should be visible only if not input
      // XXX How to not show it on last field?
      border-bottom: 1px solid rgba(0, 0, 0, 0.15);
    }
  }
*/
}
.gadget-content div[data-gadget-scope='m'] {
  animation: fadein 0.2s ease-out;
}
.gadget-content input[type='submit'] {
  padding: 6pt;
  margin-top: 30pt;
  margin-right: 12pt;
  background-color: #FF6600;
  color: #FFFFFF;
  border-radius: 0.325em;
  border-width: 1px;
  border-style: solid;
  border-color: #FF6600;
  min-width: 8em;
  box-shadow: 0 2px 4px rgba(0, 0, 0, 0.2);
  text-shadow: 0 1px 2px rgba(0, 0, 0, 0.2);
}
.gadget-content input[type='submit']:hover,
.gadget-content input[type='submit']:focus {
  background-color: #ff8533;
}
.gadget-content input[type='submit']:active {
  background-color: #ffa366;
}
.gadget-content button[name='action_update'] {
  padding: 6pt;
  margin-top: 30pt;
  margin-right: 12pt;
  background-color: #767676;
  color: #FFFFFF;
  border-radius: 0.325em;
  border-width: 1px;
  border-style: solid;
  border-color: #767676;
  min-width: 8em;
  box-shadow: 0 2px 4px rgba(0, 0, 0, 0.2);
}
.gadget-content button[name='action_update']:hover,
.gadget-content button[name='action_update']:focus {
  background-color: #909090;
}
.gadget-content button[name='action_update']:active {
  background-color: #a9a9a9;
}
.gadget-content .sign_in_with_google {
  height: 46px;
  width: 191px;
  display: inline-block;
  overflow: hidden;
  margin-top: 6pt;
}
.gadget-content .sign_in_with_google img:hover {
  margin-top: -46px;
}
.gadget-content .sign_in_with_google img:active {
  margin-top: -92px;
}
@media not screen and (max-width: 85em) {
  div[data-role='page']:not(.desktop-panel-hidden) .gadget-content {
    margin-left: 180pt;
  }
}
@media only screen and (min-width: 45em) and (max-width: 85em) {
  .gadget-content {
    padding-top: 7em;
  }
}
@media not screen and (min-width: 45em) {
  .gadget-content {
    padding: 6pt;
    padding-top: 6em;
  }
}
.gadget-content .field_container > div > div > div.ui-field-contain {
  padding: 3pt 0;
}
.gadget-content .field_container > div > div > div.ui-field-contain > div {
  width: 100%;
}
.gadget-content .field_container > div > div.horizontal_align_form_box > .ui-field-contain {
  padding: 0;
}
.gadget-content .field_container > div > div.horizontal_align_form_box .field_container > div {
  display: flex;
}
.gadget-content .field_container > div > div.horizontal_align_form_box .field_container > div > div {
  flex: 1;
}
.gadget-content .ui-content-header-plain {
  font-size: 150%;
}
.gadget-content .worklist-empty {
  max-width: 442px;
  /* original size of the embedded image */
  width: 100%;
  /* smaller screens than 442px will show full-width box */
  margin: auto;
  /* center with known width */
  text-align: center;
}
.gadget-content .worklist-empty h2 {
  font-size: 300%;
  /* copy behaviour of previously used .first-loader */
  margin-bottom: 0.5em;
}
.gadget-content .worklist-empty img {
  width: 100%;
  /* height will be computed automatically and proportionally */
}
.gadget-content ul.document-listview:not(:last-of-type) {
  margin-bottom: 12pt;
}
.gadget-content ul.document-listview:first-child {
  margin-top: 6pt;
}
.gadget-content ul.document-listview li {
  border-color: rgba(0, 0, 0, 0.3);
  border-width: 1px;
  border-style: solid;
  border-bottom-style: none;
  box-shadow: 0 1px 3px rgba(0, 0, 0, 0.15);
  background-color: #FFFFFF;
}
.gadget-content ul.document-listview li:not(.ui-li-has-count) a:after {
  font-family: FontAwesome;
  content: "\f0da";
  text-align: right;
  float: right;
  position: absolute;
  right: 12pt;
}
.gadget-content ul.document-listview li a {
  display: block;
  position: relative;
  padding: 6pt 12pt;
  padding-right: 24pt;
  color: #222222;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
}
.gadget-content ul.document-listview li a:hover,
.gadget-content ul.document-listview li a:active {
  background-color: #e0e0e0;
}
.gadget-content ul.document-listview li:first-child {
  border-top-left-radius: 0.325em;
  border-top-right-radius: 0.325em;
}
.gadget-content ul.document-listview li:last-child {
  border-bottom-left-radius: 0.325em;
  border-bottom-right-radius: 0.325em;
  border-bottom-style: solid;
}
.gadget-content ul.document-listview li .ui-li-count {
  float: right;
  padding: 0 6pt;
  border: 1px solid rgba(0, 0, 0, 0.14);
  border-radius: 0.325em;
  position: absolute;
  right: 6pt;
}
@media only screen and (min-width: 45em) and (max-width: 85em), not screen and (max-width: 85em) {
  .gadget-content .left,
  .gadget-content .right {
    vertical-align: top;
    display: inline-block;
    width: 50%;
  }
  .gadget-content .right {
    padding-left: 24pt;
  }
}
.gadget-content .ui-field-contain > label:not(.is-invalid) {
  color: hsl(0, 0%, 42%);
}
.gadget-content .required > .ui-field-contain > label {
  font-weight: bold;
  color: #1F1F1F;
}
.gadget-content .invisible > .ui-field-contain > label {
  display: none;
}
.gadget-content .page {
  background-color: #FFFFFF;
}
@media only screen and (min-width: 45em) and (max-width: 85em), not screen and (max-width: 85em) {
  .gadget-content .ui-field-contain {
    display: flex;
  }
  .gadget-content .ui-field-contain > label {
    flex: 1;
  }
  .gadget-content .ui-field-contain > label + div {
    flex: 3;
  }
}
@media only screen and (min-width: 45em) and (max-width: 85em), not screen and (max-width: 85em) {
  .gadget-content .center .ui-field-contain > label + div {
    flex: 7;
  }
}
.gadget-content .bottom iframe {
  height: 80vh;
}
.gadget-content .bottom div[data-gadget-url$="gadget_erp5_field_listbox.html"] iframe {
  height: 30vh;
}
.gadget-content form .ui-field-contain {
  position: relative;
}
.gadget-content form .ui-field-contain > span {
  animation: fadein 0.2s ease-out;
}
@media not screen and (max-width: 85em), only screen and (min-width: 45em) and (max-width: 85em) {
  .gadget-content form .ui-field-contain > span {
    background-color: #FF6600;
    color: #f8fff3;
    text-shadow: 0 1px 2px rgba(0, 0, 0, 0.2);
    left: 25%;
    position: absolute;
    bottom: 110%;
    white-space: pre;
    padding: 6pt;
    border-radius: 0.325em;
    width: auto;
    z-index: 1001;
  }
  .gadget-content form .ui-field-contain > span:before {
    position: absolute;
    top: 100%;
    left: 2em;
    display: inline-block;
    border-right: 6pt solid transparent;
    border-top: 6pt solid #FF6600;
    border-left: 6pt solid transparent;
    content: '';
  }
}
@media not screen and (min-width: 45em) {
  .gadget-content form .ui-field-contain > span {
    margin-left: 6pt;
    color: #FF6600;
  }
}
/**********************************************
* Gadget: HTML5 input field
**********************************************/
.gadget-content .ui-field-contain .ui-input-has-appendinx,
.gadget-content .ui-field-contain .ui-input-has-prependinx {
  display: flex;
}
.gadget-content .ui-field-contain .ui-input-has-appendinx i,
.gadget-content .ui-field-contain .ui-input-has-prependinx i {
  display: block;
  padding: 3pt;
  color: #777777;
  font-weight: 400;
}
/**********************************************
* Gadget: relation field
**********************************************/
.relation-input {
  display: flex;
}
.relation-input a,
.relation-input button {
  width: 24pt;
  overflow: hidden;
  text-indent: -9999px;
  white-space: nowrap;
  display: block;
  padding-top: 3pt;
  padding-bottom: 3pt;
}
.relation-input a::before,
.relation-input button::before {
  float: left;
  text-indent: 0;
  margin-left: 6pt;
}
.relation-input div {
  position: relative;
  width: 100%;
}
.relation-input ul {
  position: absolute;
  display: block;
  width: 100%;
  z-index: 501;
}
.relation-input ul li {
  cursor: pointer;
  background-color: #393939;
  color: #FFFFFF;
  padding: 3pt;
  padding-left: 6pt;
  display: block;
  text-overflow: ellipsis;
  overflow: hidden;
  white-space: nowrap;
}
.relation-input ul li::before {
  padding-right: 6pt;
}
.relation-input ul .active {
  background-color: #1F1F1F;
}
.relation-input .ui-icon-warning {
  color: #FF6600;
  opacity: 1;
}
/**********************************************
* Gadget: datetime field
**********************************************/
.datetimefield {
  display: flex;
}
.datetimefield div[data-gadget-scope=INPUT] {
  flex: 2;
}
.datetimefield div[data-gadget-scope=SELECT] {
  flex: 1;
}
/**********************************************
* Gadget: float/integer field
**********************************************/
.floatfield input,
.integerfield input {
  text-align: right;
  white-space: nowrap;
}
/**********************************************
* Listbox
**********************************************/
div[data-gadget-scope='erp5_searchfield'] {
  padding-top: 6pt;
  padding-bottom: 6pt;
}
div[data-gadget-scope='erp5_searchfield'] .ui-input-text {
  display: flex;
}
div[data-gadget-scope='erp5_searchfield'] .ui-input-text div[data-gadget-scope='input'] {
  width: 100%;
}
div[data-gadget-scope='erp5_searchfield'] button {
  padding: 3pt;
}
div[data-gadget-scope='erp5_searchfield'] div.search_parsed_value {
  display: flex;
  flex-direction: row;
}
div[data-gadget-scope='erp5_searchfield'] div.search_parsed_value button {
  border: 1px solid rgba(0, 0, 0, 0.14);
  border-radius: 0.325em;
  background-color: #FFFFFF;
  margin-right: 6pt;
  max-width: 6em;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
  -o-text-overflow: ellipsis;
  -ms-text-overflow: ellipsis;
}
.document_table .ui-table-header {
  display: flex;
  padding-top: 3pt;
  padding-bottom: 3pt;
}
@media not screen and (min-width: 45em) {
  .document_table .ui-table-header {
    border-bottom: 2px solid #085078;
  }
}
.document_table .ui-table-header h1 {
  color: hsl(0, 0%, 42%);
  flex: 2;
  align-self: flex-end;
}
@media not screen and (min-width: 45em) {
  .document_table .ui-table-header h1 span.listboxloader {
    display: none;
  }
  .document_table .ui-table-header h1 span.listboxloader.ui-icon-spinner {
    display: initial;
  }
}
.document_table .ui-table-header button {
  padding: 3pt;
  border: 1px solid rgba(0, 0, 0, 0.14);
  border-radius: 0.325em;
  background-color: #FFFFFF;
  margin-right: 6pt;
}
.document_table .ui-table-header button:last-of-type {
  margin-right: 0;
}
@media not screen and (min-width: 45em) {
  .document_table .ui-table-header button {
    width: 2em;
    overflow: hidden;
    text-indent: -9999px;
    white-space: nowrap;
  }
}
.document_table .ui-table-header button::before {
  margin-right: 6pt;
}
@media not screen and (min-width: 45em) {
  .document_table .ui-table-header button::before {
    float: left;
    text-indent: 0;
  }
}
div.invisible_header .document_table .ui-table-header {
  display: none;
}
.document_table table {
  width: 100%;
  text-align: left;
}
.document_table table th,
.document_table table td {
  vertical-align: middle;
  padding: 3pt;
}
.document_table table thead,
.document_table table tfoot {
  background-color: #085078;
  color: #FFFFFF;
}
.document_table table thead a,
.document_table table tfoot a {
  color: #FFFFFF;
  text-decoration: underline;
}
.document_table table thead tr th,
.document_table table tfoot tr th {
  padding: 6pt 3pt;
}
.document_table table thead tr th.ui-icon::before,
.document_table table tfoot tr th.ui-icon::before {
  margin-right: 6pt;
}
@media not screen and (min-width: 45em) {
  .document_table table thead,
  .document_table table tfoot {
    display: none;
  }
}
.document_table table a {
  color: #1F1F1F;
}
.document_table table .floatfield p,
.document_table table .integerfield p {
  text-align: right;
  white-space: nowrap;
}
.document_table table tbody {
  animation: fadein 0.2s ease-out;
  background-color: #FFFFFF;
}
.document_table table tbody tr:nth-child(even) {
  background-color: rgba(230, 230, 230, 0.65);
}
.document_table table tbody tr:hover,
.document_table table tbody tr:active {
  background-color: #e0e0e0;
}
@media not screen and (max-width: 85em), only screen and (min-width: 45em) and (max-width: 85em) {
  .document_table table tbody td:not(:last-child) {
    border-right: 1px solid rgba(0, 0, 0, 0.14902);
  }
}
@media not screen and (min-width: 45em) {
  .document_table table tbody tr {
    display: block;
    overflow: hidden;
    width: 100%;
    position: relative;
    height: 4em;
    line-height: 2em;
  }
  .document_table table tbody tr td *,
  .document_table table tbody tr th * {
    display: inline;
  }
  .document_table table tbody tr td:first-child,
  .document_table table tbody tr th:first-child {
    display: inline-block;
    width: 100%;
  }
  .document_table table tbody tr td:first-child a,
  .document_table table tbody tr th:first-child a,
  .document_table table tbody tr td:first-child label,
  .document_table table tbody tr th:first-child label {
    position: absolute;
    width: 100%;
    top: 0;
    bottom: 0;
    overflow: hidden;
    white-space: nowrap;
    text-overflow: ellipsis;
    padding-top: 6pt;
  }
  .document_table table tbody tr td:first-child a:after,
  .document_table table tbody tr th:first-child a:after {
    font-family: FontAwesome;
    content: "\f0da";
    font-size: 1.25em;
    position: absolute;
    right: 6pt;
    top: 50%;
    margin-top: -0.75em;
    border-radius: 0.5em;
    width: 1em;
    text-align: center;
    line-height: 1em;
  }
  .document_table table tbody tr td:first-child ~ th,
  .document_table table tbody tr th:first-child ~ th,
  .document_table table tbody tr td:first-child ~ td,
  .document_table table tbody tr th:first-child ~ td {
    font-size: 0.8em;
    display: inline;
    word-break: break-word;
  }
  .document_table table tbody tr td:first-child ~ th a,
  .document_table table tbody tr th:first-child ~ th a,
  .document_table table tbody tr td:first-child ~ td a,
  .document_table table tbody tr th:first-child ~ td a,
  .document_table table tbody tr td:first-child ~ th label,
  .document_table table tbody tr th:first-child ~ th label,
  .document_table table tbody tr td:first-child ~ td label,
  .document_table table tbody tr th:first-child ~ td label {
    pointer-events: none;
    color: hsl(0, 0%, 42%);
  }
  .document_table table tbody tr td:first-child ~ th:not(:last-child) a:not(:empty):after,
  .document_table table tbody tr th:first-child ~ th:not(:last-child) a:not(:empty):after,
  .document_table table tbody tr td:first-child ~ td:not(:last-child) a:not(:empty):after,
  .document_table table tbody tr th:first-child ~ td:not(:last-child) a:not(:empty):after,
  .document_table table tbody tr td:first-child ~ th:not(:last-child) label:not(:empty):after,
  .document_table table tbody tr th:first-child ~ th:not(:last-child) label:not(:empty):after,
  .document_table table tbody tr td:first-child ~ td:not(:last-child) label:not(:empty):after,
  .document_table table tbody tr th:first-child ~ td:not(:last-child) label:not(:empty):after {
    content: " ~ ";
  }
}
.document_table nav {
  display: flex;
  padding-top: 6pt;
  border-top: 2px solid rgba(0, 0, 0, 0.14902);
}
.document_table nav span {
  opacity: 0.3;
  flex: 2;
  text-align: right;
  float: right;
}
.document_table nav a {
  padding: 6pt;
  border: 1px solid rgba(0, 0, 0, 0.14);
  border-radius: 0.325em;
  background-color: #FFFFFF;
  margin-right: 6pt;
}
.document_table nav a::before {
  margin-right: 6pt;
}
.document_table nav a:hover,
.document_table nav a:active {
  background-color: #e0e0e0;
}
.document_table nav a:last-of-type {
  margin-right: 0;
}
.document_table nav a:hover,
.document_table nav a:active {
  background-color: #e0e0e0;
}
@media not screen and (min-width: 45em) {
  .document_table nav a {
    overflow: hidden;
    text-indent: -9999px;
    white-space: nowrap;
  }
}
@media not screen and (min-width: 45em) {
  .document_table nav a::before {
    float: left;
    text-indent: 6pt;
  }
}
div.invisible_footer .document_table nav {
  display: none;
}
.document_table tbody tr td div {
  width: 100%;
}
.document_table tbody tr td:first-child input.hide_element[type="checkbox"] {
  display: none;
}
.document_table tbody tr td:first-child input.hide_element[type="checkbox"] + label::before {
  width: 1em;
  display: inline-block;
  font-weight: bold;
  content: '☐';
  color: rgba(0, 0, 0, 0.3);
  margin-right: 6pt;
}
.document_table tbody tr td:first-child input.hide_element[type="checkbox"]:checked + label::before {
  color: #1F1F1F;
  content: '☒';
}
.document_table tbody tr td label {
  cursor: pointer;
}
/**********************************************
* Radio field
**********************************************/
div[data-gadget-url$="gadget_erp5_field_radio.html"] {
  display: inline-grid;
}
div[data-gadget-url$="gadget_erp5_field_radio.html"] label {
  display: inline;
}
/**********************************************
* Notification
**********************************************/
div[data-gadget-scope='notification'] {
  position: fixed;
  z-index: 99999;
  bottom: 12pt;
  right: -192pt;
  transition: transform 200ms ease-out;
  transform: translate3d(0, 0, 0);
}
div[data-gadget-scope='notification'].visible {
  transform: translate3d(-216pt, 0, 0);
}
div[data-gadget-scope='notification'].visible .error {
  animation: pulseerrormobile 1s ease-in infinite;
}
@media not screen and (max-width: 85em) {
  div[data-gadget-scope='notification'].visible .error {
    animation: pulseerrordesktop 1s ease-in infinite;
  }
}
@media not screen and (min-width: 45em) {
  div[data-gadget-scope='notification'].visible {
    transform: translate3d(-198pt, 0, 0);
  }
}
div[data-gadget-scope='notification'] button {
  text-align: left;
  width: 180pt;
  padding: 12pt;
  color: #f8fff3;
  text-shadow: 0 1px 2px rgba(0, 0, 0, 0.2);
  border-radius: 0.325em;
}
div[data-gadget-scope='notification'] button.success {
  background-color: #37A419;
}
div[data-gadget-scope='notification'] button.error {
  background-color: #FF6600;
}
@keyframes pulseerrormobile {
  0% {
    box-shadow: 0 0 0 0 rgba(255, 102, 0, 0.6);
  }
  100% {
    box-shadow: 0 0 0 12pt rgba(255, 102, 0, 0);
  }
}
@keyframes pulseerrordesktop {
  0% {
    box-shadow: 0 0 0 0 rgba(255, 102, 0, 0.6);
  }
  100% {
    box-shadow: 0 0 0 100pt rgba(255, 102, 0, 0);
  }
}
/**********************************************
* JQM
**********************************************/
.ui-disabled {
  opacity: 0.3;
  cursor: default;
  pointer-events: none;
}
.ui-screen-hidden {
  display: none;
}
/**********************************************
* First loader
**********************************************/
.first-loader {
  position: fixed;
  left: 50%;
  top: 50%;
  transform: translate(-50%, -50%);
  font-size: 300%;
  animation-name: fadein;
  animation-duration: 0.2s;
  animation-delay: 0.3s;
  animation-fill-mode: both;
  animation-timing-function: ease-in;
}
/**********************************************
* Keyframes
**********************************************/
@keyframes spin {
  from {
    transform: rotate(0deg);
  }
  to {
    transform: rotate(360deg);
  }
}
@keyframes fadein {
  from {
    opacity: 0;
  }
  to {
    opacity: 1;
  }
}
/**********************************************
* Desktop Panel Hidden
**********************************************/
@media not screen and (max-width: 85em) {
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header {
    margin-left: 0;
    transform: translate3d(0, 0, 0);
  }
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header h1 > span {
    padding-left: 16pt;
  }
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header h1 > span::before {
    margin-right: 14pt;
  }
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header h1 a::before,
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header h1 button::before {
    text-align: center;
  }
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header > .ui-btn-right button,
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header > .ui-btn-right a {
    border-left: 1px solid rgba(255, 255, 255, 0.55);
  }
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header .ui-controlgroup-controls button,
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header .ui-controlgroup-controls a {
    background-color: #085078;
    width: 8em;
  }
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header .ui-controlgroup-controls button.ui-icon-warning,
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header .ui-controlgroup-controls a.ui-icon-warning {
    background-color: #FF6600;
  }
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header .ui-subheader {
    flex: 1 100%;
    text-align: center;
  }
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header ul > li {
    flex: 1;
    border-left: 1px solid rgba(0, 0, 0, 0.55);
  }
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header ul > li:first-child {
    border-left: none;
  }
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header ul > li a {
    display: block;
  }
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header ul > li a {
    padding-top: 3pt;
    padding-bottom: 3pt;
    line-height: inherit;
    white-space: nowrap;
    overflow: hidden;
  }
  .desktop-panel-hidden div[data-gadget-scope="header"] .ui-header ul > li a::before {
    font-size: 1.2em;
    display: block;
  }
  .desktop-panel-hidden .gadget-content {
    padding-top: 7em;
  }
  .desktop-panel-hidden div[data-gadget-scope='panel'] {
    transform: translate3d(-186pt, 0, 0);
  }
  .desktop-panel-hidden div[data-gadget-scope='panel'].visible {
    transform: translate3d(0, 0, 0);
  }
}
/**********************************************
* Maximize
**********************************************/
div[data-gadget-scope='maximize'] button {
  padding: 6pt;
  border: 1px solid rgba(0, 0, 0, 0.14);
  border-radius: 0.325em;
  background-color: #FFFFFF;
}
div[data-gadget-scope='maximize'] button::before {
  margin-right: 6pt;
}
div[data-gadget-scope='maximize'] button:hover,
div[data-gadget-scope='maximize'] button:active {
  background-color: #e0e0e0;
}
.editor-maximize div[data-gadget-scope='editor'] {
  position: fixed;
  z-index: 1500;
  left: 0;
  top: 30pt;
  width: 100vw;
  height: calc(100vh -  30pt );
}
.editor-maximize div[data-gadget-scope='editor'] iframe {
  height: 100%;
}
div[data-gadget-scope="babylonjs"] .fullscreen {
  position: fixed;
  left: 0;
  top: 30pt;
}
div[data-gadget-scope="babylonjs"] .fullscreen-background {
  position: fixed;
  left: 0;
  top: 30pt;
  background-color: #FFFFFF;
  width: 100vw;
  height: calc(100vh -  30pt );
}
/**********************************************
* Front page
**********************************************/
ul.ui-list-grid {
  display: flex;
  align-items: flex-start;
  flex-wrap: wrap;
}
ul.ui-list-grid > li {
  flex-shrink: 1;
  vertical-align: top;
  padding: 3pt;
  border: 1px solid rgba(0, 0, 0, 0.3);
  border-radius: 0.325em;
  box-shadow: 2px 2px 2px rgba(0, 0, 0, 0.15);
  background-color: #FFFFFF;
}
@media not screen and (min-width: 45em) {
  ul.ui-list-grid > li {
    flex-basis: 100%;
    margin-bottom: 6pt;
  }
}
@media only screen and (min-width: 45em) and (max-width: 85em), not screen and (max-width: 85em) {
  ul.ui-list-grid > li {
    margin-right: 24pt;
    margin-bottom: 24pt;
    flex-basis: 20em;
  }
}
ul.ui-list-grid > li h2 {
  font-weight: 700;
  background: #085078;
  color: #FFFFFF;
  padding: 6pt;
}
ul.ui-list-grid > li li {
  padding: 3pt 6pt;
}
ul.ui-list-grid > li button {
  color: #2FA2E4;
  text-decoration: none;
}
ul.ui-list-grid > li button[href=""] {
  color: #1F1F1F;
}
ul.ui-list-grid > li button:hover {
  text-decoration: underline;
}
ul.ui-list-grid > li button:focus {
  outline-offset: -2px;
  outline: 2px solid #3388cc;
}
/**********************************************
* Developer mode actions
**********************************************/
.edit-form,
.edit-form-action,
.edit-field,
.translate-title,
.translate-description {
  margin-left: 3pt;
  color: hsl(0, 0%, 42%);
}
.translate-description {
  border: 1px solid rgba(0, 0, 0, 0.3);
}
/**********************************************
* Icons
**********************************************/
.ui-btn-icon-top::before,
.ui-btn-icon-left::before,
.ui-btn-icon-right::before,
.ui-icon::before,
.ui-btn-icon-notext::before {
  font-family: FontAwesome;
  display: inline-block;
  content: "";
}
.ui-icon-spinner {
  opacity: 1;
}
.ui-icon-spinner::before {
  content: "\f110";
  animation: spin 0.5s infinite linear;
}
/* Font Awesome uses the Unicode Private Use Area (PUA) to ensure screen
   readers do not read off random characters that represent icons */
.ui-icon-glass::before {
  content: "\f000";
}
.ui-icon-music::before {
  content: "\f001";
}
.ui-icon-search::before {
  content: "\f002";
}
.ui-icon-envelope-o::before {
  content: "\f003";
}
.ui-icon-heart::before {
  content: "\f004";
}
.ui-icon-star::before {
  content: "\f005";
}
.ui-icon-star-o::before {
  content: "\f006";
}
.ui-icon-user::before {
  content: "\f007";
}
.ui-icon-film::before {
  content: "\f008";
}
.ui-icon-th-large::before {
  content: "\f009";
}
.ui-icon-th::before {
  content: "\f00a";
}
.ui-icon-th-list::before {
  content: "\f00b";
}
.ui-icon-check::before {
  content: "\f00c";
}
.ui-icon-times::before,
.ui-icon-delete::before {
  content: "\f00d";
}
.ui-icon-search-plus::before {
  content: "\f00e";
}
.ui-icon-search-minus::before {
  content: "\f010";
}
.ui-icon-power-off::before {
  content: "\f011";
}
.ui-icon-signal::before {
  content: "\f012";
}
.ui-icon-gear::before,
.ui-icon-cog::before {
  content: "\f013";
}
.ui-icon-trash-o::before {
  content: "\f014";
}
.ui-icon-home::before {
  content: "\f015";
}
.ui-icon-file-o::before {
  content: "\f016";
}
.ui-icon-clock-o::before {
  content: "\f017";
}
.ui-icon-road::before {
  content: "\f018";
}
.ui-icon-download::before {
  content: "\f019";
}
.ui-icon-arrow-circle-o-down::before {
  content: "\f01a";
}
.ui-icon-arrow-circle-o-up::before {
  content: "\f01b";
}
.ui-icon-inbox::before {
  content: "\f01c";
}
.ui-icon-play-circle-o::before {
  content: "\f01d";
}
.ui-icon-rotate-right::before,
.ui-icon-repeat::before {
  content: "\f01e";
}
.ui-icon-refresh::before {
  content: "\f021";
}
.ui-icon-list-alt::before {
  content: "\f022";
}
.ui-icon-lock::before {
  content: "\f023";
}
.ui-icon-flag::before {
  content: "\f024";
}
.ui-icon-headphones::before {
  content: "\f025";
}
.ui-icon-volume-off::before {
  content: "\f026";
}
.ui-icon-volume-down::before {
  content: "\f027";
}
.ui-icon-volume-up::before {
  content: "\f028";
}
.ui-icon-qrcode::before {
  content: "\f029";
}
.ui-icon-barcode::before {
  content: "\f02a";
}
.ui-icon-tag::before {
  content: "\f02b";
}
.ui-icon-tags::before {
  content: "\f02c";
}
.ui-icon-book::before {
  content: "\f02d";
}
.ui-icon-bookmark::before {
  content: "\f02e";
}
.ui-icon-print::before {
  content: "\f02f";
}
.ui-icon-camera::before {
  content: "\f030";
}
.ui-icon-font::before {
  content: "\f031";
}
.ui-icon-bold::before {
  content: "\f032";
}
.ui-icon-italic::before {
  content: "\f033";
}
.ui-icon-text-height::before {
  content: "\f034";
}
.ui-icon-text-width::before {
  content: "\f035";
}
.ui-icon-align-left::before {
  content: "\f036";
}
.ui-icon-align-center::before {
  content: "\f037";
}
.ui-icon-align-right::before {
  content: "\f038";
}
.ui-icon-align-justify::before {
  content: "\f039";
}
.ui-icon-list::before {
  content: "\f03a";
}
.ui-icon-dedent::before,
.ui-icon-outdent::before {
  content: "\f03b";
}
.ui-icon-indent::before {
  content: "\f03c";
}
.ui-icon-video-camera::before {
  content: "\f03d";
}
.ui-icon-photo::before,
.ui-icon-image::before,
.ui-icon-picture-o::before {
  content: "\f03e";
}
.ui-icon-pencil::before {
  content: "\f040";
}
.ui-icon-map-marker::before {
  content: "\f041";
}
.ui-icon-adjust::before {
  content: "\f042";
}
.ui-icon-tint::before {
  content: "\f043";
}
.ui-icon-edit::before,
.ui-icon-pencil-square-o::before {
  content: "\f044";
}
.ui-icon-share-square-o::before {
  content: "\f045";
}
.ui-icon-check-square-o::before,
.ui-icon-checkbox-on::before,
.ui-checkbox-on::before {
  content: "\f046";
}
.ui-icon-arrows::before {
  content: "\f047";
}
.ui-icon-step-backward::before {
  content: "\f048";
}
.ui-icon-fast-backward::before {
  content: "\f049";
}
.ui-icon-backward::before {
  content: "\f04a";
}
.ui-icon-play::before {
  content: "\f04b";
}
.ui-icon-pause::before {
  content: "\f04c";
}
.ui-icon-stop::before {
  content: "\f04d";
}
.ui-icon-forward::before {
  content: "\f04e";
}
.ui-icon-fast-forward::before {
  content: "\f050";
}
.ui-icon-step-forward::before {
  content: "\f051";
}
.ui-icon-eject::before {
  content: "\f052";
}
.ui-icon-chevron-left::before {
  content: "\f053";
}
.ui-icon-chevron-right::before {
  content: "\f054";
}
.ui-icon-plus-circle::before {
  content: "\f055";
}
.ui-icon-minus-circle::before {
  content: "\f056";
}
.ui-icon-times-circle::before {
  content: "\f057";
}
.ui-icon-check-circle::before {
  content: "\f058";
}
.ui-icon-question-circle::before {
  content: "\f059";
}
.ui-icon-info-circle::before {
  content: "\f05a";
}
.ui-icon-crosshairs::before {
  content: "\f05b";
}
.ui-icon-times-circle-o::before {
  content: "\f05c";
}
.ui-icon-check-circle-o::before,
.ui-icon-radio-off::before,
.ui-radio-on::before {
  content: "\f05d";
}
.ui-icon-ban::before {
  content: "\f05e";
}
.ui-icon-arrow-left::before {
  content: "\f060";
}
.ui-icon-arrow-right::before {
  content: "\f061";
}
.ui-icon-arrow-up::before {
  content: "\f062";
}
.ui-icon-arrow-down::before {
  content: "\f063";
}
.ui-icon-mail-forward::before,
.ui-icon-share::before {
  content: "\f064";
}
.ui-icon-expand::before {
  content: "\f065";
}
.ui-icon-compress::before {
  content: "\f066";
}
.ui-icon-plus::before {
  content: "\f067";
}
.ui-icon-minus::before {
  content: "\f068";
}
.ui-icon-asterisk::before {
  content: "\f069";
}
.ui-icon-exclamation-circle::before {
  content: "\f06a";
}
.ui-icon-gift::before {
  content: "\f06b";
}
.ui-icon-leaf::before {
  content: "\f06c";
}
.ui-icon-fire::before {
  content: "\f06d";
}
.ui-icon-eye::before {
  content: "\f06e";
}
.ui-icon-eye-slash::before {
  content: "\f070";
}
.ui-icon-warning::before {
  content: "\f071";
}
.ui-icon-low-vision::before {
  content: "\f2a8";
}
.ui-icon-exclamation-triangle::before {
  content: "\f071";
}
.ui-icon-plane::before {
  content: "\f072";
}
.ui-icon-calendar::before {
  content: "\f073";
}
.ui-icon-random::before {
  content: "\f074";
}
.ui-icon-comment::before {
  content: "\f075";
}
.ui-icon-magnet::before {
  content: "\f076";
}
.ui-icon-chevron-up::before {
  content: "\f077";
}
.ui-icon-chevron-down::before {
  content: "\f078";
}
.ui-icon-retweet::before {
  content: "\f079";
}
.ui-icon-shopping-cart::before {
  content: "\f07a";
}
.ui-icon-folder::before {
  content: "\f07b";
}
.ui-icon-folder-open::before {
  content: "\f07c";
}
.ui-icon-arrows-v::before {
  content: "\f07d";
}
.ui-icon-arrows-h::before {
  content: "\f07e";
}
.ui-icon-bar-chart-o::before {
  content: "\f080";
}
.ui-icon-twitter-square::before {
  content: "\f081";
}
.ui-icon-facebook-square::before {
  content: "\f082";
}
.ui-icon-camera-retro::before {
  content: "\f083";
}
.ui-icon-key::before {
  content: "\f084";
}
.ui-icon-gears::before,
.ui-icon-cogs::before {
  content: "\f085";
}
.ui-icon-comments::before {
  content: "\f086";
}
.ui-icon-thumbs-o-up::before {
  content: "\f087";
}
.ui-icon-thumbs-o-down::before {
  content: "\f088";
}
.ui-icon-star-half::before {
  content: "\f089";
}
.ui-icon-heart-o::before {
  content: "\f08a";
}
.ui-icon-sign-out::before {
  content: "\f08b";
}
.ui-icon-linkedin-square::before {
  content: "\f08c";
}
.ui-icon-thumb-tack::before {
  content: "\f08d";
}
.ui-icon-external-link::before {
  content: "\f08e";
}
.ui-icon-sign-in::before {
  content: "\f090";
}
.ui-icon-trophy::before {
  content: "\f091";
}
.ui-icon-github-square::before {
  content: "\f092";
}
.ui-icon-upload::before {
  content: "\f093";
}
.ui-icon-lemon-o::before {
  content: "\f094";
}
.ui-icon-phone::before {
  content: "\f095";
}
.ui-icon-square-o::before,
.ui-icon-checkbox-off::before,
.ui-checkbox-off::before {
  content: "\f096";
}
.ui-icon-bookmark-o::before {
  content: "\f097";
}
.ui-icon-phone-square::before {
  content: "\f098";
}
.ui-icon-twitter::before {
  content: "\f099";
}
.ui-icon-facebook::before {
  content: "\f09a";
}
.ui-icon-github::before {
  content: "\f09b";
}
.ui-icon-unlock::before {
  content: "\f09c";
}
.ui-icon-credit-card::before {
  content: "\f09d";
}
.ui-icon-rss::before {
  content: "\f09e";
}
.ui-icon-hdd-o::before {
  content: "\f0a0";
}
.ui-icon-bullhorn::before {
  content: "\f0a1";
}
.ui-icon-bell::before {
  content: "\f0f3";
}
.ui-icon-certificate::before {
  content: "\f0a3";
}
.ui-icon-hand-o-right::before {
  content: "\f0a4";
}
.ui-icon-hand-o-left::before {
  content: "\f0a5";
}
.ui-icon-hand-o-up::before {
  content: "\f0a6";
}
.ui-icon-hand-o-down::before {
  content: "\f0a7";
}
.ui-icon-arrow-circle-left::before {
  content: "\f0a8";
}
.ui-icon-arrow-circle-right::before {
  content: "\f0a9";
}
.ui-icon-arrow-circle-up::before {
  content: "\f0aa";
}
.ui-icon-arrow-circle-down::before {
  content: "\f0ab";
}
.ui-icon-globe::before {
  content: "\f0ac";
}
.ui-icon-wrench::before {
  content: "\f0ad";
}
.ui-icon-tasks::before {
  content: "\f0ae";
}
.ui-icon-filter::before {
  content: "\f0b0";
}
.ui-icon-briefcase::before {
  content: "\f0b1";
}
.ui-icon-arrows-alt::before {
  content: "\f0b2";
}
.ui-icon-group::before,
.ui-icon-users::before {
  content: "\f0c0";
}
.ui-icon-chain::before,
.ui-icon-link::before {
  content: "\f0c1";
}
.ui-icon-cloud::before {
  content: "\f0c2";
}
.ui-icon-flask::before {
  content: "\f0c3";
}
.ui-icon-cut::before,
.ui-icon-scissors::before {
  content: "\f0c4";
}
.ui-icon-copy::before,
.ui-icon-files-o::before {
  content: "\f0c5";
}
.ui-icon-paperclip::before {
  content: "\f0c6";
}
.ui-icon-save::before,
.ui-icon-floppy-o::before {
  content: "\f0c7";
}
.ui-icon-square::before {
  content: "\f0c8";
}
.ui-icon-navicon::before,
.ui-icon-reorder::before,
.ui-icon-bars::before {
  content: "\f0c9";
}
.ui-icon-list-ul::before {
  content: "\f0ca";
}
.ui-icon-list-ol::before {
  content: "\f0cb";
}
.ui-icon-strikethrough::before {
  content: "\f0cc";
}
.ui-icon-underline::before {
  content: "\f0cd";
}
.ui-icon-table::before {
  content: "\f0ce";
}
.ui-icon-magic::before {
  content: "\f0d0";
}
.ui-icon-truck::before {
  content: "\f0d1";
}
.ui-icon-pinterest::before {
  content: "\f0d2";
}
.ui-icon-pinterest-square::before {
  content: "\f0d3";
}
.ui-icon-google-plus-square::before {
  content: "\f0d4";
}
.ui-icon-google-plus::before {
  content: "\f0d5";
}
.ui-icon-money::before {
  content: "\f0d6";
}
.ui-icon-caret-down::before,
.ui-icon-carat-d::before {
  content: "\f0d7";
}
.ui-icon-caret-up::before,
hmtl .ui-icon-carat-u::before {
  content: "\f0d8";
}
.ui-icon-caret-left::before,
.ui-icon-carat-l::before {
  content: "\f0d9";
}
.ui-icon-caret-right::before,
.ui-icon-carat-r::before {
  content: "\f0da";
}
.ui-icon-columns::before {
  content: "\f0db";
}
.ui-icon-unsorted::before,
.ui-icon-sort::before {
  content: "\f0dc";
}
.ui-icon-sort-down::before,
.ui-icon-sort-desc::before {
  content: "\f0dd";
}
.ui-icon-sort-up::before,
.ui-icon-sort-asc::before {
  content: "\f0de";
}
.ui-icon-envelope::before {
  content: "\f0e0";
}
.ui-icon-linkedin::before {
  content: "\f0e1";
}
.ui-icon-rotate-left::before,
.ui-icon-undo::before {
  content: "\f0e2";
}
.ui-icon-legal::before,
.ui-icon-gavel::before {
  content: "\f0e3";
}
.ui-icon-dashboard::before,
.ui-icon-tachometer::before {
  content: "\f0e4";
}
.ui-icon-comment-o::before {
  content: "\f0e5";
}
.ui-icon-comments-o::before {
  content: "\f0e6";
}
.ui-icon-flash::before,
.ui-icon-bolt::before {
  content: "\f0e7";
}
.ui-icon-sitemap::before {
  content: "\f0e8";
}
.ui-icon-umbrella::before {
  content: "\f0e9";
}
.ui-icon-paste::before,
.ui-icon-clipboard::before {
  content: "\f0ea";
}
.ui-icon-lightbulb-o::before {
  content: "\f0eb";
}
.ui-icon-exchange::before {
  content: "\f0ec";
}
.ui-icon-cloud-download::before {
  content: "\f0ed";
}
.ui-icon-cloud-upload::before {
  content: "\f0ee";
}
.ui-icon-user-md::before {
  content: "\f0f0";
}
.ui-icon-stethoscope::before {
  content: "\f0f1";
}
.ui-icon-suitcase::before {
  content: "\f0f2";
}
.ui-icon-bell-o::before {
  content: "\f0a2";
}
.ui-icon-coffee::before {
  content: "\f0f4";
}
.ui-icon-cutlery::before {
  content: "\f0f5";
}
.ui-icon-file-text-o::before {
  content: "\f0f6";
}
.ui-icon-building-o::before {
  content: "\f0f7";
}
.ui-icon-hospital-o::before {
  content: "\f0f8";
}
.ui-icon-ambulance::before {
  content: "\f0f9";
}
.ui-icon-medkit::before {
  content: "\f0fa";
}
.ui-icon-fighter-jet::before {
  content: "\f0fb";
}
.ui-icon-beer::before {
  content: "\f0fc";
}
.ui-icon-h-square::before {
  content: "\f0fd";
}
.ui-icon-plus-square::before {
  content: "\f0fe";
}
.ui-icon-angle-double-left::before {
  content: "\f100";
}
.ui-icon-angle-double-right::before {
  content: "\f101";
}
.ui-icon-angle-double-up::before {
  content: "\f102";
}
.ui-icon-angle-double-down::before {
  content: "\f103";
}
.ui-icon-angle-left::before {
  content: "\f104";
}
.ui-icon-angle-right::before {
  content: "\f105";
}
.ui-icon-angle-up::before {
  content: "\f106";
}
.ui-icon-angle-down::before {
  content: "\f107";
}
.ui-icon-desktop::before {
  content: "\f108";
}
.ui-icon-laptop::before {
  content: "\f109";
}
.ui-icon-tablet::before {
  content: "\f10a";
}
.ui-icon-mobile-phone::before,
.ui-icon-mobile::before {
  content: "\f10b";
}
.ui-icon-circle-o::before,
.ui-radio-off::before {
  content: "\f10c";
}
.ui-icon-quote-left::before {
  content: "\f10d";
}
.ui-icon-quote-right::before {
  content: "\f10e";
}
.ui-icon-circle::before {
  content: "\f111";
}
.ui-icon-mail-reply::before,
.ui-icon-reply::before {
  content: "\f112";
}
.ui-icon-github-alt::before {
  content: "\f113";
}
.ui-icon-folder-o::before {
  content: "\f114";
}
.ui-icon-folder-open-o::before {
  content: "\f115";
}
.ui-icon-smile-o::before {
  content: "\f118";
}
.ui-icon-frown-o::before {
  content: "\f119";
}
.ui-icon-meh-o::before {
  content: "\f11a";
}
.ui-icon-gamepad::before {
  content: "\f11b";
}
.ui-icon-keyboard-o::before {
  content: "\f11c";
}
.ui-icon-flag-o::before {
  content: "\f11d";
}
.ui-icon-flag-checkered::before {
  content: "\f11e";
}
.ui-icon-terminal::before {
  content: "\f120";
}
.ui-icon-code::before {
  content: "\f121";
}
.ui-icon-mail-reply-all::before,
.ui-icon-reply-all::before {
  content: "\f122";
}
.ui-icon-star-half-empty::before,
.ui-icon-star-half-full::before,
.ui-icon-star-half-o::before {
  content: "\f123";
}
.ui-icon-location-arrow::before {
  content: "\f124";
}
.ui-icon-crop::before {
  content: "\f125";
}
.ui-icon-code-fork::before {
  content: "\f126";
}
.ui-icon-unlink::before,
.ui-icon-chain-broken::before {
  content: "\f127";
}
.ui-icon-question::before {
  content: "\f128";
}
.ui-icon-info::before {
  content: "\f129";
}
.ui-icon-exclamation::before {
  content: "\f12a";
}
.ui-icon-superscript::before {
  content: "\f12b";
}
.ui-icon-subscript::before {
  content: "\f12c";
}
.ui-icon-eraser::before {
  content: "\f12d";
}
.ui-icon-puzzle-piece::before {
  content: "\f12e";
}
.ui-icon-microphone::before {
  content: "\f130";
}
.ui-icon-microphone-slash::before {
  content: "\f131";
}
.ui-icon-shield::before {
  content: "\f132";
}
.ui-icon-calendar-o::before {
  content: "\f133";
}
.ui-icon-fire-extinguisher::before {
  content: "\f134";
}
.ui-icon-rocket::before {
  content: "\f135";
}
.ui-icon-maxcdn::before {
  content: "\f136";
}
.ui-icon-chevron-circle-left::before {
  content: "\f137";
}
.ui-icon-chevron-circle-right::before {
  content: "\f138";
}
.ui-icon-chevron-circle-up::before {
  content: "\f139";
}
.ui-icon-chevron-circle-down::before {
  content: "\f13a";
}
.ui-icon-html5::before {
  content: "\f13b";
}
.ui-icon-css3::before {
  content: "\f13c";
}
.ui-icon-anchor::before {
  content: "\f13d";
}
.ui-icon-unlock-alt::before {
  content: "\f13e";
}
.ui-icon-bullseye::before {
  content: "\f140";
}
.ui-icon-ellipsis-h::before {
  content: "\f141";
}
.ui-icon-ellipsis-v::before {
  content: "\f142";
}
.ui-icon-rss-square::before {
  content: "\f143";
}
.ui-icon-play-circle::before {
  content: "\f144";
}
.ui-icon-ticket::before {
  content: "\f145";
}
.ui-icon-minus-square::before {
  content: "\f146";
}
.ui-icon-minus-square-o::before {
  content: "\f147";
}
.ui-icon-level-up::before {
  content: "\f148";
}
.ui-icon-level-down::before {
  content: "\f149";
}
.ui-icon-check-square::before {
  content: "\f14a";
}
.ui-icon-pencil-square::before {
  content: "\f14b";
}
.ui-icon-external-link-square::before {
  content: "\f14c";
}
.ui-icon-share-square::before {
  content: "\f14d";
}
.ui-icon-compass::before {
  content: "\f14e";
}
.ui-icon-toggle-down::before,
.ui-icon-caret-square-o-down::before {
  content: "\f150";
}
.ui-icon-toggle-up::before,
.ui-icon-caret-square-o-up::before {
  content: "\f151";
}
.ui-icon-toggle-right::before,
.ui-icon-caret-square-o-right::before {
  content: "\f152";
}
.ui-icon-euro::before,
.ui-icon-eur::before {
  content: "\f153";
}
.ui-icon-gbp::before {
  content: "\f154";
}
.ui-icon-dollar::before,
.ui-icon-usd::before {
  content: "\f155";
}
.ui-icon-rupee::before,
.ui-icon-inr::before {
  content: "\f156";
}
.ui-icon-cny::before,
.ui-icon-rmb::before,
.ui-icon-yen::before,
.ui-icon-jpy::before {
  content: "\f157";
}
.ui-icon-ruble::before,
.ui-icon-rouble::before,
.ui-icon-rub::before {
  content: "\f158";
}
.ui-icon-won::before,
.ui-icon-krw::before {
  content: "\f159";
}
.ui-icon-bitcoin::before,
.ui-icon-btc::before {
  content: "\f15a";
}
.ui-icon-file::before {
  content: "\f15b";
}
.ui-icon-file-text::before {
  content: "\f15c";
}
.ui-icon-sort-alpha-asc::before {
  content: "\f15d";
}
.ui-icon-sort-alpha-desc::before {
  content: "\f15e";
}
.ui-icon-sort-amount-asc::before {
  content: "\f160";
}
.ui-icon-sort-amount-desc::before {
  content: "\f161";
}
.ui-icon-sort-numeric-asc::before {
  content: "\f162";
}
.ui-icon-sort-numeric-desc::before {
  content: "\f163";
}
.ui-icon-thumbs-up::before {
  content: "\f164";
}
.ui-icon-thumbs-down::before {
  content: "\f165";
}
.ui-icon-youtube-square::before {
  content: "\f166";
}
.ui-icon-youtube::before {
  content: "\f167";
}
.ui-icon-xing::before {
  content: "\f168";
}
.ui-icon-xing-square::before {
  content: "\f169";
}
.ui-icon-youtube-play::before {
  content: "\f16a";
}
.ui-icon-dropbox::before {
  content: "\f16b";
}
.ui-icon-stack-overflow::before {
  content: "\f16c";
}
.ui-icon-instagram::before {
  content: "\f16d";
}
.ui-icon-flickr::before {
  content: "\f16e";
}
.ui-icon-adn::before {
  content: "\f170";
}
.ui-icon-bitbucket::before {
  content: "\f171";
}
.ui-icon-bitbucket-square::before {
  content: "\f172";
}
.ui-icon-tumblr::before {
  content: "\f173";
}
.ui-icon-tumblr-square::before {
  content: "\f174";
}
.ui-icon-long-arrow-down::before {
  content: "\f175";
}
.ui-icon-long-arrow-up::before {
  content: "\f176";
}
.ui-icon-long-arrow-left::before {
  content: "\f177";
}
.ui-icon-long-arrow-right::before {
  content: "\f178";
}
.ui-icon-apple::before {
  content: "\f179";
}
.ui-icon-windows::before {
  content: "\f17a";
}
.ui-icon-android::before {
  content: "\f17b";
}
.ui-icon-linux::before {
  content: "\f17c";
}
.ui-icon-dribbble::before {
  content: "\f17d";
}
.ui-icon-skype::before {
  content: "\f17e";
}
.ui-icon-foursquare::before {
  content: "\f180";
}
.ui-icon-trello::before {
  content: "\f181";
}
.ui-icon-female::before {
  content: "\f182";
}
.ui-icon-male::before {
  content: "\f183";
}
.ui-icon-gittip::before {
  content: "\f184";
}
.ui-icon-sun-o::before {
  content: "\f185";
}
.ui-icon-moon-o::before {
  content: "\f186";
}
.ui-icon-archive::before {
  content: "\f187";
}
.ui-icon-bug::before {
  content: "\f188";
}
.ui-icon-vk::before {
  content: "\f189";
}
.ui-icon-weibo::before {
  content: "\f18a";
}
.ui-icon-renren::before {
  content: "\f18b";
}
.ui-icon-pagelines::before {
  content: "\f18c";
}
.ui-icon-stack-exchange::before {
  content: "\f18d";
}
.ui-icon-arrow-circle-o-right::before {
  content: "\f18e";
}
.ui-icon-arrow-circle-o-left::before {
  content: "\f190";
}
.ui-icon-toggle-left::before,
.ui-icon-caret-square-o-left::before {
  content: "\f191";
}
.ui-icon-dot-circle-o::before {
  content: "\f192";
}
.ui-icon-wheelchair::before {
  content: "\f193";
}
.ui-icon-vimeo-square::before {
  content: "\f194";
}
.ui-icon-turkish-lira::before,
.ui-icon-try::before {
  content: "\f195";
}
.ui-icon-plus-square-o::before {
  content: "\f196";
}
.ui-icon-space-shuttle::before {
  content: "\f197";
}
.ui-icon-slack::before {
  content: "\f198";
}
.ui-icon-envelope-square::before {
  content: "\f199";
}
.ui-icon-wordpress::before {
  content: "\f19a";
}
.ui-icon-openid::before {
  content: "\f19b";
}
.ui-icon-institution::before,
.ui-icon-bank::before,
.ui-icon-university::before {
  content: "\f19c";
}
.ui-icon-mortar-board::before,
.ui-icon-graduation-cap::before {
  content: "\f19d";
}
.ui-icon-yahoo::before {
  content: "\f19e";
}
.ui-icon-google::before {
  content: "\f1a0";
}
.ui-icon-reddit::before {
  content: "\f1a1";
}
.ui-icon-reddit-square::before {
  content: "\f1a2";
}
.ui-icon-stumbleupon-circle::before {
  content: "\f1a3";
}
.ui-icon-stumbleupon::before {
  content: "\f1a4";
}
.ui-icon-delicious::before {
  content: "\f1a5";
}
.ui-icon-digg::before {
  content: "\f1a6";
}
.ui-icon-pied-piper-square::before,
.ui-icon-pied-piper::before {
  content: "\f1a7";
}
.ui-icon-pied-piper-alt::before {
  content: "\f1a8";
}
.ui-icon-drupal::before {
  content: "\f1a9";
}
.ui-icon-joomla::before {
  content: "\f1aa";
}
.ui-icon-language::before {
  content: "\f1ab";
}
.ui-icon-fax::before {
  content: "\f1ac";
}
.ui-icon-building::before {
  content: "\f1ad";
}
.ui-icon-child::before {
  content: "\f1ae";
}
.ui-icon-paw::before {
  content: "\f1b0";
}
.ui-icon-spoon::before {
  content: "\f1b1";
}
.ui-icon-cube::before {
  content: "\f1b2";
}
.ui-icon-cubes::before {
  content: "\f1b3";
}
.ui-icon-behance::before {
  content: "\f1b4";
}
.ui-icon-behance-square::before {
  content: "\f1b5";
}
.ui-icon-steam::before {
  content: "\f1b6";
}
.ui-icon-steam-square::before {
  content: "\f1b7";
}
.ui-icon-recycle::before {
  content: "\f1b8";
}
.ui-icon-automobile::before,
.ui-icon-car::before {
  content: "\f1b9";
}
.ui-icon-cab::before,
.ui-icon-taxi::before {
  content: "\f1ba";
}
.ui-icon-tree::before {
  content: "\f1bb";
}
.ui-icon-spotify::before {
  content: "\f1bc";
}
.ui-icon-deviantart::before {
  content: "\f1bd";
}
.ui-icon-soundcloud::before {
  content: "\f1be";
}
.ui-icon-database::before {
  content: "\f1c0";
}
.ui-icon-file-pdf-o::before {
  content: "\f1c1";
}
.ui-icon-file-word-o::before {
  content: "\f1c2";
}
.ui-icon-file-excel-o::before {
  content: "\f1c3";
}
.ui-icon-file-powerpoint-o::before {
  content: "\f1c4";
}
.ui-icon-file-photo-o::before,
.ui-icon-file-picture-o::before,
.ui-icon-file-image-o::before {
  content: "\f1c5";
}
.ui-icon-file-zip-o::before,
.ui-icon-file-archive-o::before {
  content: "\f1c6";
}
.ui-icon-file-sound-o::before,
.ui-icon-file-audio-o::before {
  content: "\f1c7";
}
.ui-icon-file-movie-o::before,
.ui-icon-file-video-o::before {
  content: "\f1c8";
}
.ui-icon-file-code-o::before {
  content: "\f1c9";
}
.ui-icon-vine::before {
  content: "\f1ca";
}
.ui-icon-codepen::before {
  content: "\f1cb";
}
.ui-icon-jsfiddle::before {
  content: "\f1cc";
}
.ui-icon-life-bouy::before,
.ui-icon-life-saver::before,
.ui-icon-support::before,
.ui-icon-life-ring::before {
  content: "\f1cd";
}
.ui-icon-circle-o-notch::before {
  content: "\f1ce";
}
.ui-icon-ra::before,
.ui-icon-rebel::before {
  content: "\f1d0";
}
.ui-icon-ge::before,
.ui-icon-empire::before {
  content: "\f1d1";
}
.ui-icon-git-square::before {
  content: "\f1d2";
}
.ui-icon-git::before {
  content: "\f1d3";
}
.ui-icon-hacker-news::before {
  content: "\f1d4";
}
.ui-icon-tencent-weibo::before {
  content: "\f1d5";
}
.ui-icon-qq::before {
  content: "\f1d6";
}
.ui-icon-wechat::before,
.ui-icon-weixin::before {
  content: "\f1d7";
}
.ui-icon-send::before,
.ui-icon-paper-plane::before {
  content: "\f1d8";
}
.ui-icon-send-o::before,
.ui-icon-paper-plane-o::before {
  content: "\f1d9";
}
.ui-icon-history::before {
  content: "\f1da";
}
.ui-icon-circle-thin::before {
  content: "\f1db";
}
.ui-icon-header::before {
  content: "\f1dc";
}
.ui-icon-paragraph::before {
  content: "\f1dd";
}
.ui-icon-sliders::before {
  content: "\f1de";
}
.ui-icon-share-alt::before {
  content: "\f1e0";
}
.ui-icon-share-alt-square::before {
  content: "\f1e1";
}
.ui-icon-bomb::before {
  content: "\f1e2";
}
.ui-icon-clone::before {
  content: "\f24d";
}